import uuid
from datetime import datetime
from unittest.mock import patch

import pytest
from ckan import model
from ckan.tests import factories

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentCategory,
    ResourceCommentSummary,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import (
    Utilization,
    UtilizationComment,
    UtilizationCommentCategory,
)
from ckanext.feedback.services.management import comments
from ckanext.feedback.services.resource import comment, summary


def get_registered_utilization(resource_id):
    return (
        session.query(
            Utilization.id,
            Utilization.approval,
            Utilization.approved,
            Utilization.approval_user_id,
        )
        .filter(Utilization.resource_id == resource_id)
        .all()
    )


def get_registered_utilization_comment(utilization_id):
    return (
        session.query(
            UtilizationComment.id,
            UtilizationComment.utilization_id,
            UtilizationComment.category,
            UtilizationComment.content,
            UtilizationComment.created,
            UtilizationComment.approval,
            UtilizationComment.approved,
            UtilizationComment.approval_user_id,
        )
        .filter(UtilizationComment.utilization_id == utilization_id)
        .all()
    )


def register_utilization(id, resource_id, title, description, approval):
    utilization = Utilization(
        id=id,
        resource_id=resource_id,
        title=title,
        description=description,
        approval=approval,
    )
    session.add(utilization)


def register_utilization_comment(
    id, utilization_id, category, content, created, approval, approved, approval_user_id
):
    utilization_comment = UtilizationComment(
        id=id,
        utilization_id=utilization_id,
        category=category,
        content=content,
        created=created,
        approval=approval,
        approved=approved,
        approval_user_id=approval_user_id,
    )
    session.add(utilization_comment)


def get_resource_comment_summary(resource_id):
    resource_comment_summary = (
        session.query(ResourceCommentSummary)
        .filter(ResourceCommentSummary.resource_id == resource_id)
        .first()
    )
    return resource_comment_summary


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestComments:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_utilization_comments(self):
        id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'
        created = datetime.now()
        approved = datetime.now()

        assert comments.get_utilization_comments(id) == 0

        register_utilization(id, resource['id'], title, description, True)
        register_utilization_comment(
            comment_id, id, category, content, created, True, approved, None
        )
        session.commit()

        assert comments.get_utilization_comments(id) == 1

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilizations(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        another_utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'

        comment_id = str(uuid.uuid4())
        another_comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'
        created = datetime.now()
        approved = datetime.now()

        register_utilization(utilization_id, resource['id'], title, description, True)
        register_utilization(
            another_utilization_id, resource['id'], title, description, True
        )

        register_utilization_comment(
            comment_id, utilization_id, category, content, created, True, approved, None
        )

        register_utilization_comment(
            another_comment_id,
            another_utilization_id,
            category,
            content,
            created,
            True,
            approved,
            None,
        )

        session.commit()

        assert len(comments.get_utilizations([comment_id, another_comment_id])) == 2
        assert (
            comments.get_utilizations([comment_id, another_comment_id])[0].id
            == utilization_id
        )
        assert (
            comments.get_utilizations([comment_id, another_comment_id])[1].id
            == another_utilization_id
        )

    def test_get_organization(self):
        organization_dict = factories.Organization()
        assert (
            comments.get_organization(organization_dict['id']).id
            == organization_dict['id']
        )

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    @patch('ckanext.feedback.services.management.comments.get_utilization_comments')
    @patch('ckanext.feedback.services.management.comments.session.bulk_update_mappings')
    def test_refresh_utilizations_comments(
        self, mock_mappings, mock_get_utilization_comments
    ):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        another_utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'

        register_utilization(utilization_id, resource['id'], title, description, True)
        register_utilization(
            another_utilization_id, resource['id'], title, description, True
        )

        session.commit()

        mock_get_utilization_comments.return_value = 0
        comments.refresh_utilizations_comments(
            [
                get_registered_utilization(resource['id'])[0],
                get_registered_utilization(resource['id'])[1],
            ]
        )

        expected_args = (
            Utilization,
            [
                {
                    'id': utilization_id,
                    'comment': 0,
                    'updated': datetime.now(),
                },
                {
                    'id': another_utilization_id,
                    'comment': 0,
                    'updated': datetime.now(),
                },
            ],
        )

        assert mock_get_utilization_comments.call_count == 2
        assert mock_mappings.call_args[0] == expected_args

    def test_get_resource_comment_summaries(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])
        another_resource = factories.Resource(package_id=dataset['id'])

        category = ResourceCommentCategory.QUESTION

        comment.create_resource_comment(resource['id'], category, 'test content 1', 1)
        comment.create_resource_comment(
            another_resource['id'], category, 'test content 2', 5
        )
        summary.create_resource_summary(resource['id'])
        summary.create_resource_summary(another_resource['id'])

        resource_comment = comment.get_resource_comments(resource['id'], None)
        another_resource_comment = comment.get_resource_comments(
            another_resource['id'], None
        )

        comment.approve_resource_comment(resource_comment[0].id, None)
        comment.approve_resource_comment(another_resource_comment[0].id, None)

        summary.refresh_resource_summary(resource['id'])
        summary.refresh_resource_summary(another_resource['id'])

        session.commit()

        comment_id_list = [resource_comment[0].id, another_resource_comment[0].id]

        resource_comment_summaries = comments.get_resource_comment_summaries(
            comment_id_list
        )

        assert len(resource_comment_summaries) == 2
        assert resource_comment_summaries[0].comment == 1
        assert resource_comment_summaries[1].comment == 1

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    @patch('ckanext.feedback.services.management.comments.session.bulk_update_mappings')
    def test_refresh_resource_comments(self, mock_mappings):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])
        another_resource = factories.Resource(package_id=dataset['id'])

        category = ResourceCommentCategory.QUESTION

        comment.create_resource_comment(resource['id'], category, 'test content 1', 1)
        comment.create_resource_comment(
            another_resource['id'], category, 'test content 2', 5
        )
        summary.create_resource_summary(resource['id'])
        summary.create_resource_summary(another_resource['id'])

        resource_comment = comment.get_resource_comments(resource['id'], None)
        another_resource_comment = comment.get_resource_comments(
            another_resource['id'], None
        )

        comment.approve_resource_comment(resource_comment[0].id, None)
        comment.approve_resource_comment(another_resource_comment[0].id, None)

        summary.refresh_resource_summary(resource['id'])
        summary.refresh_resource_summary(another_resource['id'])

        session.commit()

        resource_comment_summary = get_resource_comment_summary(resource['id'])
        another_resource_comment_summary = get_resource_comment_summary(
            another_resource['id']
        )

        resource_comment_summaries = [
            resource_comment_summary,
            another_resource_comment_summary,
        ]

        comments.refresh_resources_comments(resource_comment_summaries)

        expected_mapping = [
            {
                'id': resource_comment_summary.id,
                'comment': 1,
                'rating_comment': 1,
                'rating': 1,
                'updated': datetime.now(),
            },
            {
                'id': another_resource_comment_summary.id,
                'comment': 1,
                'rating_comment': 1,
                'rating': 5,
                'updated': datetime.now(),
            },
        ]

        assert mock_mappings.call_args[0] == (ResourceCommentSummary, expected_mapping)

        comments.delete_resource_comments(
            [resource_comment[0].id, another_resource_comment[0].id]
        )
        comments.refresh_resources_comments(resource_comment_summaries)
        session.commit()
        expected_mapping = [
            {
                'id': resource_comment_summary.id,
                'comment': 0,
                'rating_comment': 0,
                'rating': 0,
                'updated': datetime.now(),
            },
            {
                'id': another_resource_comment_summary.id,
                'comment': 0,
                'rating_comment': 0,
                'rating': 0,
                'updated': datetime.now(),
            },
        ]

        assert mock_mappings.call_args[0] == (ResourceCommentSummary, expected_mapping)

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    @patch('ckanext.feedback.services.management.comments.session.bulk_update_mappings')
    def test_approve_utilization_comments(self, mock_mappings):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'

        comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'
        created = datetime.now()

        register_utilization(utilization_id, resource['id'], title, description, True)
        register_utilization_comment(
            comment_id, utilization_id, category, content, created, False, None, None
        )

        session.commit()

        comment_id_list = [comment_id]

        comments.approve_utilization_comments(comment_id_list, None)

        expected_args = (
            UtilizationComment,
            [
                {
                    'id': comment_id,
                    'approval': True,
                    'approved': datetime.now(),
                    'approval_user_id': None,
                }
            ],
        )

        assert mock_mappings.call_args[0] == expected_args

    def test_delete_utilization_comments(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'

        comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'
        created = datetime.now()

        register_utilization(utilization_id, resource['id'], title, description, True)
        register_utilization_comment(
            comment_id, utilization_id, category, content, created, False, None, None
        )

        session.commit()

        utilization_comment = get_registered_utilization_comment(utilization_id)
        assert len(utilization_comment) == 1

        comments.delete_utilization_comments([comment_id])

        utilization_comment = get_registered_utilization_comment(utilization_id)
        assert len(utilization_comment) == 0

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    @patch('ckanext.feedback.services.management.comments.session.bulk_update_mappings')
    def test_approve_resource_comments(self, mock_mappings):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        category = ResourceCommentCategory.QUESTION

        comment.create_resource_comment(resource['id'], category, 'test content 1', 1)
        resource_comment = comment.get_resource_comments(resource['id'], None)

        session.commit()

        comment_id_list = [resource_comment[0].id]

        comments.approve_resource_comments(comment_id_list, None)

        expected_args = (
            ResourceComment,
            [
                {
                    'id': resource_comment[0].id,
                    'approval': True,
                    'approved': datetime.now(),
                    'approval_user_id': None,
                }
            ],
        )

        assert mock_mappings.call_args[0] == expected_args

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_delete_resource_comments(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        category = ResourceCommentCategory.QUESTION

        comment.create_resource_comment(resource['id'], category, 'test content 1', 1)
        resource_comment = comment.get_resource_comments(resource['id'], None)

        session.commit()

        comment_id_list = [resource_comment[0].id]
        assert len(resource_comment) == 1

        comments.delete_resource_comments(comment_id_list)

        resource_comment = comment.get_resource_comments(resource['id'], None)
        assert len(resource_comment) == 0



import logging

import ckan.model as model
from ckan.common import _, config, current_user, g, request
from ckan.lib import helpers
from ckan.logic import get_action
from ckan.plugins import toolkit
from flask import redirect, url_for

import ckanext.feedback.services.resource.comment as comment_service
import ckanext.feedback.services.utilization.details as detail_service
import ckanext.feedback.services.utilization.edit as edit_service
import ckanext.feedback.services.utilization.registration as registration_service
import ckanext.feedback.services.utilization.search as search_service
import ckanext.feedback.services.utilization.summary as summary_service
import ckanext.feedback.services.utilization.validate as validate_service
from ckanext.feedback.controllers.pagination import get_pagination_value
from ckanext.feedback.models.session import session
from ckanext.feedback.services.common.check import (
    check_administrator,
    has_organization_admin_role,
    is_organization_admin,
)
from ckanext.feedback.services.common.send_mail import send_email
from ckanext.feedback.services.recaptcha.check import is_recaptcha_verified

log = logging.getLogger(__name__)


class UtilizationController:
    # Render HTML pages
    # utilization/search
    @staticmethod
    def search():
        id = request.args.get('id', '')
        keyword = request.args.get('keyword', '')
        org_name = request.args.get('organization', '')

        unapproved_status = request.args.get('waiting', 'on')
        approval_status = request.args.get('approval', 'on')

        page, limit, offset, pager_url = get_pagination_value('utilization.search')

        # If the login user is not an admin, display only approved utilizations
        approval = True
        admin_owner_orgs = None
        if not isinstance(current_user, model.User):
            # If the user is not login, display only approved utilizations
            approval = True
        elif current_user.sysadmin:
            # If the user is an admin, display all utilizations
            approval = None
        elif is_organization_admin():
            # If the user is an organization admin, display all utilizations
            approval = None
            admin_owner_orgs = current_user.get_group_ids(
                group_type='organization', capacity='admin'
            )

        disable_keyword = request.args.get('disable_keyword', '')
        utilizations, total_count = search_service.get_utilizations(
            id,
            keyword,
            approval,
            admin_owner_orgs,
            org_name,
            limit,
            offset,
        )

        # If the organization name can be identified,
        # set it as a global variable accessible from templates.
        if id and not org_name:
            resource = comment_service.get_resource(id)
            if resource:
                org_name = resource.organization_name
            else:
                org_name = search_service.get_organization_name_from_pkg(id)
        if org_name:
            g.pkg_dict = {
                'organization': {
                    'name': org_name,
                },
            }

        return toolkit.render(
            'utilization/search.html',
            {
                'keyword': keyword,
                'disable_keyword': disable_keyword,
                'unapproved_status': unapproved_status,
                'approval_status': approval_status,
                'page': helpers.Page(
                    collection=utilizations,
                    page=page,
                    url=pager_url,
                    item_count=total_count,
                    items_per_page=limit,
                ),
            },
        )

    # utilization/new
    @staticmethod
    def new(resource_id=None, title='', description=''):
        if not resource_id:
            resource_id = request.args.get('resource_id', '')
        return_to_resource = request.args.get('return_to_resource', False)
        resource = comment_service.get_resource(resource_id)
        context = {'model': model, 'session': session, 'for_view': True}
        package = get_action('package_show')(
            context, {'id': resource.Resource.package.id}
        )
        g.pkg_dict = {'organization': {'name': resource.organization_name}}

        return toolkit.render(
            'utilization/new.html',
            {
                'pkg_dict': package,
                'return_to_resource': return_to_resource,
                'resource': resource.Resource,
            },
        )

    # utilization/new
    @staticmethod
    def create():
        package_name = request.form.get('package_name', '')
        resource_id = request.form.get('resource_id', '')
        title = request.form.get('title', '')
        url = request.form.get('url', '')
        description = request.form.get('description', '')

        url_err_msg = validate_service.validate_url(url)
        title_err_msg = validate_service.validate_title(title)
        dsc_err_msg = validate_service.validate_description(description)
        if (url and url_err_msg) or title_err_msg or dsc_err_msg:
            if title_err_msg:
                helpers.flash_error(
                    _(title_err_msg),
                    allow_html=True,
                )
            if url and url_err_msg:
                helpers.flash_error(
                    _(url_err_msg),
                    allow_html=True,
                )
            if dsc_err_msg:
                helpers.flash_error(
                    _(dsc_err_msg),
                    allow_html=True,
                )
            return toolkit.redirect_to(
                'utilization.new',
                resource_id=resource_id,
            )

        if not (resource_id and title and description):
            toolkit.abort(400)

        if not is_recaptcha_verified(request):
            helpers.flash_error(_(u'Bad Captcha. Please try again.'), allow_html=True)
            return toolkit.redirect_to(
                'utilization.new',
                resource_id=resource_id,
                title=title,
                description=description,
            )
        return_to_resource = toolkit.asbool(request.form.get('return_to_resource'))
        utilization = registration_service.create_utilization(
            resource_id, title, url, description
        )
        summary_service.create_utilization_summary(resource_id)
        utilization_id = utilization.id
        session.commit()

        try:
            resource = comment_service.get_resource(resource_id)
            send_email(
                template_name=config.get(
                    'ckan.feedback.notice.email.template_utilization'
                ),
                organization_id=resource.package.owner_org,
                subject=config.get('ckan.feedback.notice.email.subject_utilization'),
                target_name=resource.name,
                content_title=title,
                content=description,
                url=url_for(
                    'utilization.details', utilization_id=utilization_id, _external=True
                ),
            )
        except Exception:
            log.exception('Send email failed, for feedback notification.')

        helpers.flash_success(
            _(
                'Your application is complete.<br>The utilization will not be displayed'
                ' until approved by an administrator.'
            ),
            allow_html=True,
        )

        if return_to_resource:
            return redirect(
                url_for('resource.read', id=package_name, resource_id=resource_id)
            )
        else:
            return redirect(url_for('dataset.read', id=package_name))

    # utilization/<utilization_id>
    @staticmethod
    def details(utilization_id, category='', content=''):
        approval = True
        utilization = detail_service.get_utilization(utilization_id)
        if not isinstance(current_user, model.User):
            approval = True
        elif (
            has_organization_admin_role(utilization.owner_org) or current_user.sysadmin
        ):
            # if the user is an organization admin or a sysadmin, display all comments
            approval = None

        page, limit, offset, _ = get_pagination_value('utilization.details')

        comments, total_count = detail_service.get_utilization_comments(
            utilization_id, approval, limit=limit, offset=offset
        )
        categories = detail_service.get_utilization_comment_categories()
        issue_resolutions = detail_service.get_issue_resolutions(utilization_id)
        g.pkg_dict = {
            'organization': {
                'name': (
                    comment_service.get_resource(
                        utilization.resource_id
                    ).organization_name
                )
            }
        }

        return toolkit.render(
            'utilization/details.html',
            {
                'utilization_id': utilization_id,
                'utilization': utilization,
                'categories': categories,
                'issue_resolutions': issue_resolutions,
                'selected_category': category,
                'content': content,
                'page': helpers.Page(
                    collection=comments,
                    page=page,
                    item_count=total_count,
                    items_per_page=limit,
                ),
            },
        )

    # utilization/<utilization_id>/approve
    @staticmethod
    @check_administrator
    def approve(utilization_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        resource_id = detail_service.get_utilization(utilization_id).resource_id
        detail_service.approve_utilization(utilization_id, current_user.id)
        summary_service.refresh_utilization_summary(resource_id)
        session.commit()

        return redirect(url_for('utilization.details', utilization_id=utilization_id))

    # utilization/<utilization_id>/comment/new
    @staticmethod
    def create_comment(utilization_id):
        category = request.form.get('category', '')
        content = request.form.get('content', '')
        if not (category and content):
            toolkit.abort(400)

        if not is_recaptcha_verified(request):
            helpers.flash_error(_(u'Bad Captcha. Please try again.'), allow_html=True)
            return UtilizationController.details(utilization_id, category, content)

        if message := validate_service.validate_comment(content):
            helpers.flash_error(
                _(message),
                allow_html=True,
            )
            return toolkit.redirect_to(
                'utilization.details',
                utilization_id=utilization_id,
                category=category,
            )

        detail_service.create_utilization_comment(utilization_id, category, content)
        session.commit()

        try:
            utilization = detail_service.get_utilization(utilization_id)
            send_email(
                template_name=config.get(
                    'ckan.feedback.notice.email.template_utilization_comment'
                ),
                organization_id=comment_service.get_resource(
                    utilization.resource_id
                ).package.owner_org,
                subject=config.get(
                    'ckan.feedback.notice.email.subject_utilization_comment'
                ),
                target_name=utilization.title,
                category=category,
                content=content,
                url=toolkit.url_for(
                    'utilization.details', utilization_id=utilization_id, _external=True
                ),
            )
        except Exception:
            log.exception('Send email failed, for feedback notification.')

        helpers.flash_success(
            _(
                'Your comment has been sent.<br>The comment will not be displayed until'
                ' approved by an administrator.'
            ),
            allow_html=True,
        )

        return toolkit.redirect_to('utilization.details', utilization_id=utilization_id)

    # utilization/<utilization_id>/comment/<comment_id>/approve
    @staticmethod
    @check_administrator
    def approve_comment(utilization_id, comment_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        detail_service.approve_utilization_comment(comment_id, current_user.id)
        detail_service.refresh_utilization_comments(utilization_id)
        session.commit()

        return redirect(url_for('utilization.details', utilization_id=utilization_id))

    # utilization/<utilization_id>/edit
    @staticmethod
    @check_administrator
    def edit(utilization_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        utilization_details = edit_service.get_utilization_details(utilization_id)
        resource_details = edit_service.get_resource_details(
            utilization_details.resource_id
        )
        g.pkg_dict = {
            'organization': {
                'name': (
                    comment_service.get_resource(
                        utilization_details.resource_id
                    ).organization_name
                )
            }
        }

        return toolkit.render(
            'utilization/edit.html',
            {
                'utilization_details': utilization_details,
                'resource_details': resource_details,
            },
        )

    # utilization/<utilization_id>/edit
    @staticmethod
    @check_administrator
    def update(utilization_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        title = request.form.get('title', '')
        url = request.form.get('url', '')
        description = request.form.get('description', '')
        if not (title and description):
            toolkit.abort(400)

        url_err_msg = validate_service.validate_url(url)
        title_err_msg = validate_service.validate_title(title)
        dsc_err_msg = validate_service.validate_description(description)
        if (url and url_err_msg) or title_err_msg or dsc_err_msg:
            if title_err_msg:
                helpers.flash_error(
                    _(title_err_msg),
                    allow_html=True,
                )
            if url and url_err_msg:
                helpers.flash_error(
                    _(url_err_msg),
                    allow_html=True,
                )
            if dsc_err_msg:
                helpers.flash_error(
                    _(dsc_err_msg),
                    allow_html=True,
                )
            return toolkit.redirect_to(
                'utilization.edit',
                utilization_id=utilization_id,
            )

        edit_service.update_utilization(utilization_id, title, url, description)
        session.commit()

        helpers.flash_success(
            _('The utilization has been successfully updated.'),
            allow_html=True,
        )

        return redirect(url_for('utilization.details', utilization_id=utilization_id))

    # utilization/<utilization_id>/delete
    @staticmethod
    @check_administrator
    def delete(utilization_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        resource_id = detail_service.get_utilization(utilization_id).resource_id
        edit_service.delete_utilization(utilization_id)
        session.commit()
        summary_service.refresh_utilization_summary(resource_id)
        session.commit()

        helpers.flash_success(
            _('The utilization has been successfully deleted.'),
            allow_html=True,
        )

        return redirect(url_for('utilization.search'))

    # utilization/<utilization_id>/issue_resolution/new
    @staticmethod
    @check_administrator
    def create_issue_resolution(utilization_id):
        UtilizationController._check_organization_admin_role(utilization_id)
        description = request.form.get('description')
        if not description:
            toolkit.abort(400)

        detail_service.create_issue_resolution(
            utilization_id, description, current_user.id
        )
        summary_service.increment_issue_resolution_summary(utilization_id)
        session.commit()

        return redirect(url_for('utilization.details', utilization_id=utilization_id))

    @staticmethod
    def _check_organization_admin_role(utilization_id):
        utilization = detail_service.get_utilization(utilization_id)
        if (
            not has_organization_admin_role(utilization.owner_org)
            and not current_user.sysadmin
        ):
            toolkit.abort(
                404,
                _(
                    'The requested URL was not found on the server. If you entered the'
                    ' URL manually please check your spelling and try again.'
                ),
            )



from flask import Blueprint

from ckanext.feedback.controllers import utilization
from ckanext.feedback.views.error_handler import add_error_handler

blueprint = Blueprint('utilization', __name__, url_prefix='/utilization')

# Add target page URLs to rules and add each URL to the blueprint
rules = [
    (
        '/search',
        'search',
        utilization.UtilizationController.search,
        {'methods': ['GET']},
    ),
    ('/new', 'new', utilization.UtilizationController.new, {'methods': ['GET']}),
    ('/new', 'create', utilization.UtilizationController.create, {'methods': ['POST']}),
    (
        '/<utilization_id>',
        'details',
        utilization.UtilizationController.details,
        {'methods': ['GET']},
    ),
    (
        '/<utilization_id>/approve',
        'approve',
        utilization.UtilizationController.approve,
        {'methods': ['POST']},
    ),
    (
        '/<utilization_id>/comment/new',
        'create_comment',
        utilization.UtilizationController.create_comment,
        {'methods': ['POST']},
    ),
    (
        '/<utilization_id>/comment/<comment_id>/approve',
        'approve_comment',
        utilization.UtilizationController.approve_comment,
        {'methods': ['POST']},
    ),
    (
        '/<utilization_id>/edit',
        'edit',
        utilization.UtilizationController.edit,
        {'methods': ['GET']},
    ),
    (
        '/<utilization_id>/edit',
        'update',
        utilization.UtilizationController.update,
        {'methods': ['POST']},
    ),
    (
        '/<utilization_id>/delete',
        'delete',
        utilization.UtilizationController.delete,
        {'methods': ['POST']},
    ),
    (
        '/<utilization_id>/issue_resolution/new',
        'create_issue_resolution',
        utilization.UtilizationController.create_issue_resolution,
        {'methods': ['POST']},
    ),
]
for rule, endpoint, view_func, *others in rules:
    options = next(iter(others), {})
    blueprint.add_url_rule(rule, endpoint, view_func, **options)


@add_error_handler
def get_utilization_blueprint():
    return blueprint



from datetime import datetime

from ckan.model.package import Package
from ckan.model.resource import Resource

from ckanext.feedback.models.issue import IssueResolution
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import (
    Utilization,
    UtilizationComment,
    UtilizationCommentCategory,
)


# Get details from the Utilization record
def get_utilization(utilization_id):
    return (
        session.query(
            Utilization.title,
            Utilization.url,
            Utilization.description,
            Utilization.comment,
            Utilization.approval,
            Resource.name.label('resource_name'),
            Resource.id.label('resource_id'),
            Package.name.label('package_name'),
            Package.owner_org,
        )
        .join(Resource, Utilization.resource)
        .join(Package)
        .filter(Utilization.id == utilization_id)
        .first()
    )


# Approve currently displayed utilization
def approve_utilization(utilization_id, approval_user_id):
    utilization = session.query(Utilization).get(utilization_id)
    utilization.approval = True
    utilization.approved = datetime.now()
    utilization.approval_user_id = approval_user_id


# Get comments related to the Utilization record
def get_utilization_comments(
    utilization_id=None, approval=None, owner_orgs=None, limit=None, offset=None
):
    query = session.query(UtilizationComment).order_by(
        UtilizationComment.created.desc()
    )
    if utilization_id is not None:
        query = query.filter(UtilizationComment.utilization_id == utilization_id)
    if approval is not None:
        query = query.filter(UtilizationComment.approval == approval)
    if owner_orgs is not None:
        query = (
            query.join(Utilization)
            .join(Resource)
            .join(Package)
            .filter(Package.owner_org.in_(owner_orgs))
        )

    results = query.limit(limit).offset(offset).all()
    if limit is not None or offset is not None:
        total_count = query.count()
        return results, total_count
    return results


# Create comment for currently displayed utilization
def create_utilization_comment(utilization_id, category, content):
    comment = UtilizationComment(
        utilization_id=utilization_id,
        category=category,
        content=content,
    )
    session.add(comment)


# Approve selected utilization comment
def approve_utilization_comment(comment_id, approval_user_id):
    comment = session.query(UtilizationComment).get(comment_id)
    comment.approval = True
    comment.approved = datetime.now()
    comment.approval_user_id = approval_user_id


# Get comment category enum names and values
def get_utilization_comment_categories():
    return UtilizationCommentCategory


# Get issues resolved related to the Utilization record
def get_issue_resolutions(utilization_id):
    return (
        session.query(IssueResolution)
        .filter(IssueResolution.utilization_id == utilization_id)
        .order_by(IssueResolution.created.desc())
        .all()
    )


# Create issue resolution
def create_issue_resolution(utilization_id, description, creator_user_id):
    issue_resolution = IssueResolution(
        utilization_id=utilization_id,
        description=description,
        creator_user_id=creator_user_id,
    )
    session.add(issue_resolution)


# Recalculate total approved utilization comments
def refresh_utilization_comments(utilization_id):
    count = (
        session.query(UtilizationComment)
        .filter(
            UtilizationComment.utilization_id == utilization_id,
            UtilizationComment.approval,
        )
        .count()
    )
    utilization = session.query(Utilization).get(utilization_id)
    utilization.comment = count
    utilization.updated = datetime.now()



import uuid
from datetime import datetime

import ckan.tests.factories as factories
import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization
from ckanext.feedback.services.utilization.search import get_utilizations


def register_utilization(id, resource_id, title, description, approval, created):
    utilization = Utilization(
        id=id,
        resource_id=resource_id,
        title=title,
        description=description,
        approval=approval,
        created=created,
    )
    session.add(utilization)


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilizations(self):
        unapproved_org = factories.Organization(
            is_organization=True,
            name='unapproved_org_name',
            type='organization',
            title='unapproved_org',
        )
        unapproved_dataset = factories.Dataset(owner_org=unapproved_org['id'])
        unapproved_resource = factories.Resource(package_id=unapproved_dataset['id'])
        unapproved_id = str(uuid.uuid4())
        unapproved_title = 'unapproved title'

        approved_org = factories.Organization(
            is_organization=True,
            name='approved_org_name',
            type='organization',
            title='approved_org',
        )
        approved_dataset = factories.Dataset(owner_org=approved_org['id'])
        approved_resource = factories.Resource(package_id=approved_dataset['id'])
        approved_id = str(uuid.uuid4())
        approved_title = 'approved title'

        description = 'test description'

        limit = 20
        offset = 0

        register_utilization(
            unapproved_id,
            unapproved_resource['id'],
            unapproved_title,
            description,
            False,
            datetime(1999, 1, 2, 3, 4),
        )
        register_utilization(
            approved_id,
            approved_resource['id'],
            approved_title,
            description,
            True,
            datetime(2000, 1, 2, 3, 4),
        )

        unapproved_utilization = (
            unapproved_id,
            unapproved_title,
            0,
            datetime(1999, 1, 2, 3, 4),
            False,
            unapproved_resource['name'],
            unapproved_resource['id'],
            unapproved_dataset['name'],
            unapproved_org['id'],
            unapproved_org['title'],
            unapproved_org['name'],
            0,
        )

        approved_utilization = (
            approved_id,
            approved_title,
            0,
            datetime(2000, 1, 2, 3, 4),
            True,
            approved_resource['name'],
            approved_resource['id'],
            approved_dataset['name'],
            approved_org['id'],
            approved_org['title'],
            approved_org['name'],
            0,
        )

        # with no argument
        assert get_utilizations() == ([approved_utilization, unapproved_utilization], 2)

        # with package_id
        assert get_utilizations(id=unapproved_dataset['id']) == (
            [unapproved_utilization],
            1,
        )

        # with resource_id
        assert get_utilizations(id=approved_resource['id']) == (
            [approved_utilization],
            1,
        )

        # with keyword
        assert get_utilizations(keyword='unapproved') == ([unapproved_utilization], 1)

        # with approval
        assert get_utilizations(approval=True) == ([approved_utilization], 1)

        # with org_name
        assert get_utilizations(org_name=unapproved_org['name']) == (
            [unapproved_utilization],
            1,
        )

        # with organization_id
        assert get_utilizations(admin_owner_orgs=[approved_org['id']]) == (
            [approved_utilization],
            1,
        )

        # with organization_id
        assert get_utilizations(admin_owner_orgs=[unapproved_org['id']]) == (
            [
                approved_utilization,
                unapproved_utilization,
            ],
            2,
        )

        # with limit offset
        assert get_utilizations(limit=limit, offset=offset) == (
            [
                approved_utilization,
                unapproved_utilization,
            ],
            2,
        )



from flask import Blueprint

from ckanext.feedback.controllers import resource
from ckanext.feedback.views.error_handler import add_error_handler

blueprint = Blueprint('resource_comment', __name__, url_prefix='/resource_comment')

# Add target page URLs to rules and add each URL to the blueprint
rules = [
    (
        '/<resource_id>',
        'comment',
        resource.ResourceController.comment,
        {'methods': ['GET']},
    ),
    (
        '/<resource_id>/comment/new',
        'create_comment',
        resource.ResourceController.create_comment,
        {'methods': ['POST']},
    ),
    (
        '/<resource_id>/comment/approve',
        'approve_comment',
        resource.ResourceController.approve_comment,
        {'methods': ['POST']},
    ),
    (
        '/<resource_id>/comment/reply',
        'reply',
        resource.ResourceController.reply,
        {'methods': ['POST']},
    ),
]
for rule, endpoint, view_func, *others in rules:
    options = next(iter(others), {})
    blueprint.add_url_rule(rule, endpoint, view_func, **options)


@add_error_handler
def get_resource_comment_blueprint():
    return blueprint



from unittest.mock import patch

import ckan.tests.factories as factories
import pytest
from ckan import model
from ckan.plugins import toolkit

from ckanext.feedback.services.common.config import (
    CONFIG_HANDLER_PATH,
    download_handler,
    get_organization,
)

engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestCheck:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()

    def test_check_administrator(self):
        example_organization = factories.Organization(
            is_organization=True,
            name='org_name',
            type='organization',
            title='org_title',
        )

        result = get_organization(example_organization['id'])
        assert result.name == example_organization['name']

    @patch('ckanext.feedback.services.common.config.import_string')
    def test_seted_download_handler(self, mock_import_string):
        toolkit.config['ckan.feedback.download_handler'] = CONFIG_HANDLER_PATH
        download_handler()
        mock_import_string.assert_called_once_with(CONFIG_HANDLER_PATH, silent=True)



from unittest.mock import MagicMock, patch

import pytest
from ckan import model
from ckan.common import _
from ckan.model import User
from ckan.tests import factories
from flask import Flask, g

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.controllers.management import ManagementController

engine = model.repo.session.get_bind()


@pytest.fixture
def sysadmin_env():
    user = factories.SysadminWithToken()
    env = {'Authorization': user['token']}
    return env


@pytest.fixture
def user_env():
    user = factories.UserWithToken()
    env = {'Authorization': user['token']}
    return env


def mock_current_user(current_user, user):
    user_obj = model.User.get(user['name'])
    # mock current_user
    current_user.return_value = user_obj


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestManagementController:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.render')
    @patch('ckanext.feedback.controllers.management.request.args')
    @patch('ckanext.feedback.controllers.management.utilization_detail_service')
    @patch('ckanext.feedback.controllers.management.resource_comment_service')
    def test_comments_with_sysadmin(
        self,
        mock_comment_service,
        mock_detail_service,
        mock_args,
        mock_render,
        current_user,
        app,
        sysadmin_env,
    ):
        categories = ['category']
        utilization_comments = ['utilization_comment']
        resource_comments = ['resource_comment']
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        mock_comment_service.get_resource_comments.return_value = resource_comments
        mock_detail_service.get_utilization_comment_categories.return_value = categories
        mock_detail_service.get_utilization_comments.return_value = utilization_comments
        mock_args.get.return_value = 'utilization-comments'

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            ManagementController.comments()

        mock_detail_service.get_utilization_comment_categories.assert_called_once()
        mock_detail_service.get_utilization_comments.assert_called_once()
        mock_comment_service.get_resource_comments.assert_called_once()
        mock_args.get.assert_called_once_with('tab', 'utilization-comments')

        mock_render.assert_called_once_with(
            'management/comments.html',
            {
                'categories': categories,
                'utilization_comments': utilization_comments,
                'resource_comments': resource_comments,
                'tab': 'utilization-comments',
            },
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.render')
    @patch('ckanext.feedback.controllers.management.request.args')
    @patch('ckanext.feedback.controllers.management.utilization_detail_service')
    @patch('ckanext.feedback.controllers.management.resource_comment_service')
    def test_comments_with_org_admin(
        self,
        mock_comment_service,
        mock_detail_service,
        mock_args,
        mock_render,
        current_user,
        app,
        user_env,
    ):
        categories = ['category']
        utilization_comments = ['utilization_comment']
        resource_comments = ['resource_comment']
        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)

        organization_dict = factories.Organization()
        organization = model.Group.get(organization_dict['id'])

        member = model.Member(
            group=organization,
            group_id=organization_dict['id'],
            table_id=user.id,
            table_name='user',
            capacity='admin',
        )
        model.Session.add(member)
        model.Session.commit()

        mock_comment_service.get_resource_comments.return_value = resource_comments
        mock_detail_service.get_utilization_comment_categories.return_value = categories
        mock_detail_service.get_utilization_comments.return_value = utilization_comments
        mock_args.get.return_value = 'utilization-comments'

        with app.get(url='/', environ_base=user_env):
            g.userobj = current_user
            ManagementController.comments()

        mock_detail_service.get_utilization_comment_categories.assert_called_once()
        mock_detail_service.get_utilization_comments.assert_called_once()
        mock_comment_service.get_resource_comments.assert_called_once()
        mock_args.get.assert_called_once_with('tab', 'utilization-comments')

        mock_render.assert_called_once_with(
            'management/comments.html',
            {
                'categories': categories,
                'utilization_comments': utilization_comments,
                'resource_comments': resource_comments,
                'tab': 'utilization-comments',
            },
        )
        assert g.pkg_dict['organization']['name'] is not None

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management._')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.helpers.flash_success')
    @patch('ckanext.feedback.controllers.management.session.commit')
    @patch('ckanext.feedback.controllers.management.comments_service')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_approve_bulk_utilization_comments(
        self,
        mock_form,
        mock_comments_service,
        mock_session_commit,
        mock_flash_success,
        mock_url_for,
        mock_redirect,
        _,
        current_user,
        app,
        sysadmin_env,
    ):
        comments = ['comment']
        utilization = MagicMock()
        utilization.resource.package.owner_org = 'owner_org'
        utilizations = [utilization]

        mock_form.getlist.return_value = comments
        mock_comments_service.get_utilizations.return_value = utilizations
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.approve_bulk_utilization_comments()

        mock_form.getlist.assert_called_once_with('utilization-comments-checkbox')
        mock_comments_service.get_utilizations.assert_called_once_with(comments)
        mock_comments_service.approve_utilization_comments.assert_called_once_with(
            comments, user_dict['id']
        )
        mock_comments_service.refresh_utilizations_comments.assert_called_once_with(
            utilizations
        )
        mock_session_commit.assert_called_once()
        mock_flash_success.assert_called_once_with(
            f'{len(comments)} ' + _('bulk approval completed.'),
            allow_html=True,
        )
        mock_url_for.assert_called_once_with(
            'management.comments', tab='utilization-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_approve_bulk_utilization_comments_without_comment(
        self, mock_form, mock_url_for, mock_redirect, current_user, app, sysadmin_env
    ):
        mock_form.getlist.return_value = None
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.approve_bulk_utilization_comments()

        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management._')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.helpers.flash_success')
    @patch('ckanext.feedback.controllers.management.session.commit')
    @patch('ckanext.feedback.controllers.management.comments_service')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_approve_bulk_resource_comments(
        self,
        mock_form,
        mock_comments_service,
        mock_session_commit,
        mock_flash_success,
        mock_url_for,
        mock_redirect,
        _,
        current_user,
        app,
        sysadmin_env,
    ):
        comments = ['comment']
        resource_comment_summary = MagicMock()
        resource_comment_summary.resource.package.owner_org = 'owner_org'
        resource_comment_summaries = [resource_comment_summary]

        mock_form.getlist.return_value = comments
        mock_comments_service.get_resource_comment_summaries.return_value = (
            resource_comment_summaries
        )
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.approve_bulk_resource_comments()

        mock_form.getlist.assert_called_once_with('resource-comments-checkbox')
        mock_comments_service.get_resource_comment_summaries.assert_called_once_with(
            comments
        )
        mock_comments_service.approve_resource_comments.assert_called_once_with(
            comments, user_dict['id']
        )
        mock_comments_service.refresh_resources_comments.assert_called_once_with(
            resource_comment_summaries
        )
        mock_session_commit.assert_called_once()
        mock_flash_success.assert_called_once_with(
            f'{len(comments)} ' + _('bulk approval completed.'),
            allow_html=True,
        )
        mock_url_for.assert_called_once_with(
            'management.comments', tab='resource-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_approve_bulk_resource_comments_without_comment(
        self,
        mock_form,
        mock_url_for,
        mock_redirect,
        current_user,
        app,
        sysadmin_env,
    ):
        mock_form.getlist.return_value = None
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.approve_bulk_resource_comments()

        mock_url_for.assert_called_once_with(
            'management.comments', tab='resource-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management._')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.helpers.flash_success')
    @patch('ckanext.feedback.controllers.management.session.commit')
    @patch('ckanext.feedback.controllers.management.comments_service')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_delete_bulk_utilization_comments(
        self,
        mock_form,
        mock_comments_service,
        mock_session_commit,
        mock_flash_success,
        mock_url_for,
        mock_redirect,
        _,
        current_user,
        app,
        sysadmin_env,
    ):
        comments = ['comment']
        utilization = MagicMock()
        utilization.resource.package.owner_org = 'owner_org'
        utilizations = [utilization]

        mock_form.getlist.return_value = comments
        mock_comments_service.get_utilizations.return_value = utilizations
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.delete_bulk_utilization_comments()

        mock_form.getlist.assert_called_once_with('utilization-comments-checkbox')
        mock_comments_service.get_utilizations.assert_called_once_with(comments)
        mock_comments_service.delete_utilization_comments.assert_called_once_with(
            comments
        )
        mock_comments_service.refresh_utilizations_comments.assert_called_once_with(
            utilizations
        )
        mock_session_commit.assert_called_once()
        mock_flash_success.assert_called_once_with(
            f'{len(comments)} ' + _('bulk delete completed.'),
            allow_html=True,
        )
        mock_url_for.assert_called_once_with(
            'management.comments', tab='utilization-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_delete_bulk_utilization_comments_without_comment(
        self,
        mock_form,
        mock_url_for,
        mock_redirect,
        current_user,
        app,
        sysadmin_env,
    ):
        mock_form.getlist.return_value = None
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.delete_bulk_utilization_comments()

        mock_url_for.assert_called_once_with(
            'management.comments', tab='utilization-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management._')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.helpers.flash_success')
    @patch('ckanext.feedback.controllers.management.session.commit')
    @patch('ckanext.feedback.controllers.management.comments_service')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_delete_bulk_resource_comments(
        self,
        mock_form,
        mock_comments_service,
        mock_session_commit,
        mock_flash_success,
        mock_url_for,
        mock_redirect,
        _,
        current_user,
        app,
        sysadmin_env,
    ):
        comments = ['comment1', 'comment2']
        resource_comment_summary1 = MagicMock()
        resource_comment_summary2 = MagicMock()
        resource_comment_summary1.resource.package.owner_org = 'owner_org1'
        resource_comment_summary2.resource.package.owner_org = 'owner_org2'
        resource_comment_summaries = [
            resource_comment_summary1,
            resource_comment_summary2,
        ]

        mock_form.getlist.return_value = comments
        mock_comments_service.get_resource_comment_summaries.return_value = (
            resource_comment_summaries
        )
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.delete_bulk_resource_comments()

        mock_form.getlist.assert_called_once_with('resource-comments-checkbox')
        mock_comments_service.get_resource_comment_summaries.assert_called_once_with(
            comments
        )
        mock_comments_service.delete_resource_comments.assert_called_once_with(comments)
        mock_comments_service.refresh_resources_comments.assert_called_once_with(
            resource_comment_summaries
        )
        mock_session_commit.assert_called_once()
        mock_flash_success.assert_called_once_with(
            f'{len(comments)} ' + _('bulk delete completed.'),
            allow_html=True,
        )
        mock_url_for.assert_called_once_with(
            'management.comments', tab='resource-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.redirect')
    @patch('ckanext.feedback.controllers.management.url_for')
    @patch('ckanext.feedback.controllers.management.request.form')
    def test_delete_bulk_resource_comments_without_comment(
        self,
        mock_form,
        mock_url_for,
        mock_redirect,
        current_user,
        app,
        sysadmin_env,
    ):
        mock_form.getlist.return_value = None
        mock_url_for.return_value = 'url'
        mock_redirect.return_value = 'redirect_response'
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            response = ManagementController.delete_bulk_resource_comments()

        mock_url_for.assert_called_once_with(
            'management.comments', tab='resource-comments'
        )
        mock_redirect.assert_called_once_with('url')

        assert response == 'redirect_response'

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_utilization_using_sysadmin(
        self, mock_toolkit_abort, current_user
    ):
        mocked_utilization = MagicMock()
        mocked_utilization.resource.package.owner_org = 'owner_org'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user
        ManagementController._check_organization_admin_role_with_utilization(
            [mocked_utilization]
        )
        mock_toolkit_abort.assert_not_called()

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_utilization_using_org_admin(
        self, mock_toolkit_abort, current_user
    ):
        mocked_utilization = MagicMock()

        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        organization_dict = factories.Organization()
        organization = model.Group.get(organization_dict['id'])

        mocked_utilization.resource.package.owner_org = organization_dict['id']

        member = model.Member(
            group=organization,
            group_id=organization_dict['id'],
            table_id=user.id,
            table_name='user',
            capacity='admin',
        )
        model.Session.add(member)
        model.Session.commit()

        ManagementController._check_organization_admin_role_with_utilization(
            [mocked_utilization]
        )
        mock_toolkit_abort.assert_not_called()

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_utilization_using_user(
        self, mock_toolkit_abort, current_user
    ):
        mocked_utilization = MagicMock()

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        organization_dict = factories.Organization()

        mocked_utilization.resource.package.owner_org = organization_dict['id']

        ManagementController._check_organization_admin_role_with_utilization(
            [mocked_utilization]
        )
        mock_toolkit_abort.assert_called_once_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_resource_using_sysadmin(
        self, mock_toolkit_abort, current_user
    ):
        mocked_resource_comment_summary = MagicMock()
        mocked_resource_comment_summary.resource.package.owner_org = 'owner_org'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user
        ManagementController._check_organization_admin_role_with_resource(
            [mocked_resource_comment_summary]
        )
        mock_toolkit_abort.assert_not_called()

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_resource_using_org_admin(
        self, mock_toolkit_abort, current_user
    ):
        mocked_resource_comment_summary = MagicMock()

        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        organization_dict = factories.Organization()
        organization = model.Group.get(organization_dict['id'])

        mocked_resource_comment_summary.resource.package.owner_org = organization_dict[
            'id'
        ]

        member = model.Member(
            group=organization,
            group_id=organization_dict['id'],
            table_id=user.id,
            table_name='user',
            capacity='admin',
        )
        model.Session.add(member)
        model.Session.commit()

        ManagementController._check_organization_admin_role_with_resource(
            [mocked_resource_comment_summary]
        )
        mock_toolkit_abort.assert_not_called()

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.management.toolkit.abort')
    def test_check_organization_admin_role_with_resource_using_user(
        self, mock_toolkit_abort, current_user
    ):
        mocked_resource_comment_summary = MagicMock()

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        organization_dict = factories.Organization()

        mocked_resource_comment_summary.resource.package.owner_org = organization_dict[
            'id'
        ]

        ManagementController._check_organization_admin_role_with_resource(
            [mocked_resource_comment_summary]
        )
        mock_toolkit_abort.assert_called_once_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )



import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.services.resource import validate


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestComments:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_validate_with_valid_comment(self):
        example_valid_comment = 'example comment'
        result = validate.validate_comment(example_valid_comment)
        assert result is None

    def test_validate_with_invalid_comment(self):
        example_invalid_comment = 'ex'
        for _ in range(9):
            example_invalid_comment += example_invalid_comment
        result = validate.validate_comment(example_invalid_comment)
        assert result == 'Please keep the comment length below 1000'



import uuid
from datetime import datetime

import ckan.tests.factories as factories
import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.issue import IssueResolution
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import (
    Utilization,
    UtilizationComment,
    UtilizationCommentCategory,
)
from ckanext.feedback.services.utilization.details import (
    approve_utilization,
    approve_utilization_comment,
    create_issue_resolution,
    create_utilization_comment,
    get_issue_resolutions,
    get_utilization,
    get_utilization_comment_categories,
    get_utilization_comments,
    refresh_utilization_comments,
)


def get_registered_utilization(resource_id):
    return (
        session.query(
            Utilization.id,
            Utilization.approval,
            Utilization.approved,
            Utilization.approval_user_id,
        )
        .filter(Utilization.resource_id == resource_id)
        .first()
    )


def get_registered_utilization_comment(utilization_id):
    return (
        session.query(
            UtilizationComment.id,
            UtilizationComment.utilization_id,
            UtilizationComment.category,
            UtilizationComment.content,
            UtilizationComment.created,
            UtilizationComment.approval,
            UtilizationComment.approved,
            UtilizationComment.approval_user_id,
        )
        .filter(UtilizationComment.utilization_id == utilization_id)
        .all()
    )


def get_registered_issue_resolution(utilization_id):
    return (
        session.query(
            IssueResolution.utilization_id,
            IssueResolution.description,
            IssueResolution.creator_user_id,
        )
        .filter(IssueResolution.utilization_id == utilization_id)
        .first()
    )


def register_utilization(id, resource_id, title, url, description, approval):
    utilization = Utilization(
        id=id,
        resource_id=resource_id,
        title=title,
        url=url,
        description=description,
        approval=approval,
    )
    session.add(utilization)


def register_utilization_comment(
    id, utilization_id, category, content, created, approval, approved, approval_user_id
):
    utilization_comment = UtilizationComment(
        id=id,
        utilization_id=utilization_id,
        category=category,
        content=content,
        created=created,
        approval=approval,
        approved=approved,
        approval_user_id=approval_user_id,
    )
    session.add(utilization_comment)


def convert_utilization_comment_to_tuple(utilization_comment):
    return (
        utilization_comment.id,
        utilization_comment.utilization_id,
        utilization_comment.category,
        utilization_comment.content,
        utilization_comment.created,
        utilization_comment.approval,
        utilization_comment.approved,
        utilization_comment.approval_user_id,
    )


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_utilization(self):
        organization = factories.Organization()
        dataset = factories.Dataset(owner_org=organization['id'])
        resource = factories.Resource(package_id=dataset['id'])

        assert get_registered_utilization(resource['id']) is None

        id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(id, resource['id'], title, url, description, False)

        result = get_utilization(id)
        expected_utilization = (
            title,
            url,
            description,
            0,
            False,
            resource['name'],
            resource['id'],
            dataset['name'],
            organization['id'],
        )
        assert result == expected_utilization

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_approve_utilization(self):
        dataset = factories.Dataset()
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])
        test_datetime = datetime.now()

        id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(id, resource['id'], title, url, description, False)

        result = get_registered_utilization(resource['id'])
        unapproved_utilization = (id, False, None, None)
        assert result == unapproved_utilization

        approve_utilization(id, user['id'])

        result = get_registered_utilization(resource['id'])
        approved_utilization = (id, True, test_datetime, user['id'])
        assert result == approved_utilization

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilization_comments_utilization_id_and_approval_are_None(self):
        dataset = factories.Dataset()
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        created = datetime.now()
        approved = datetime.now()
        unapproved_comment_id = str(uuid.uuid4())
        category_request = UtilizationCommentCategory.REQUEST
        unapproved_content = 'unapproved content'
        register_utilization_comment(
            unapproved_comment_id,
            utilization.id,
            category_request,
            unapproved_content,
            created,
            False,
            None,
            None,
        )
        comments = get_utilization_comments(utilization.id, None)

        assert len(comments) == 1
        comment = convert_utilization_comment_to_tuple(comments[0])
        unapproved_comment = (
            unapproved_comment_id,
            utilization.id,
            category_request,
            unapproved_content,
            created,
            False,
            None,
            None,
        )
        assert comment == unapproved_comment

        approved_comment_id = str(uuid.uuid4())
        category_thank = UtilizationCommentCategory.THANK
        approved_content = 'approved content'
        register_utilization_comment(
            approved_comment_id,
            utilization.id,
            category_thank,
            approved_content,
            datetime(2001, 1, 2, 3, 4),
            True,
            approved,
            user['id'],
        )
        comments = get_utilization_comments(None, None)

        assert len(comments) == 2
        approved_result = convert_utilization_comment_to_tuple(comments[0])
        unapproved_result = convert_utilization_comment_to_tuple(comments[1])
        approved_comment = (
            approved_comment_id,
            utilization.id,
            category_thank,
            approved_content,
            datetime(2001, 1, 2, 3, 4),
            True,
            approved,
            user['id'],
        )
        assert unapproved_result == unapproved_comment
        assert approved_result == approved_comment

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilization_comments_approval_is_False(self):
        dataset = factories.Dataset()
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        created = datetime.now()
        approved = datetime.now()
        unapproved_comment_id = str(uuid.uuid4())
        approved_comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'

        register_utilization_comment(
            unapproved_comment_id,
            utilization.id,
            category,
            content,
            created,
            False,
            None,
            None,
        )
        register_utilization_comment(
            approved_comment_id,
            utilization.id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )
        comments = get_utilization_comments(utilization.id, False)

        assert len(comments) == 1
        unapproved_comment = convert_utilization_comment_to_tuple(comments[0])
        expect_unapproved_comment = (
            unapproved_comment_id,
            utilization.id,
            category,
            content,
            created,
            False,
            None,
            None,
        )
        assert unapproved_comment == expect_unapproved_comment

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilization_comments_approval_is_True(self):
        dataset = factories.Dataset()
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        created = datetime.now()
        approved = datetime.now()
        unapproved_comment_id = str(uuid.uuid4())
        approved_comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'

        register_utilization_comment(
            unapproved_comment_id,
            utilization.id,
            category,
            content,
            created,
            False,
            None,
            None,
        )
        register_utilization_comment(
            approved_comment_id,
            utilization.id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )
        comments = get_utilization_comments(utilization.id, True)

        assert len(comments) == 1
        approved_comment = convert_utilization_comment_to_tuple(comments[0])
        fake_utilization_comment_approved = (
            approved_comment_id,
            utilization.id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )
        assert approved_comment == fake_utilization_comment_approved

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilization_comments_owner_org(self):
        organization = factories.Organization()
        dataset = factories.Dataset(owner_org=organization['id'])
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        created = datetime.now()
        approved = datetime.now()
        unapproved_comment_id = str(uuid.uuid4())
        approved_comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'

        register_utilization_comment(
            unapproved_comment_id,
            utilization.id,
            category,
            content,
            created,
            False,
            None,
            None,
        )
        register_utilization_comment(
            approved_comment_id,
            utilization.id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )
        comments = get_utilization_comments(utilization.id, True, [organization['id']])

        assert len(comments) == 1
        approved_comment = convert_utilization_comment_to_tuple(comments[0])
        fake_utilization_comment_approved = (
            approved_comment_id,
            utilization.id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )
        assert approved_comment == fake_utilization_comment_approved

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_get_utilization_comments_limit_offset(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])
        limit = 20
        offset = 0

        created = datetime.now()
        unapproved_comment_id = str(uuid.uuid4())
        category_request = UtilizationCommentCategory.REQUEST
        unapproved_content = 'unapproved content'
        register_utilization_comment(
            unapproved_comment_id,
            utilization.id,
            category_request,
            unapproved_content,
            created,
            False,
            None,
            None,
        )
        comments, total_count = get_utilization_comments(
            utilization.id,
            None,
            limit=limit,
            offset=offset,
        )

        assert len(comments) == 1
        comment = convert_utilization_comment_to_tuple(comments[0])
        unapproved_comment = (
            unapproved_comment_id,
            utilization.id,
            category_request,
            unapproved_content,
            created,
            False,
            None,
            None,
        )
        assert comment == unapproved_comment

    def test_create_utilization_comment(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        category = UtilizationCommentCategory.REQUEST
        content = 'test content'
        create_utilization_comment(utilization.id, category, content)

        comments = get_registered_utilization_comment(utilization.id)
        comment = comments[0]
        assert comment.utilization_id == utilization.id
        assert comment.category == category
        assert comment.content == content
        assert comment.approval is False
        assert comment.approved is None
        assert comment.approval_user_id is None

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_approve_utilization_comment(self):
        dataset = factories.Dataset()
        user = factories.User()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(
            utilization_id, resource['id'], title, url, description, False
        )
        utilization = get_registered_utilization(resource['id'])

        created = datetime.now()
        comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'
        register_utilization_comment(
            comment_id,
            utilization.id,
            category,
            content,
            created,
            False,
            None,
            None,
        )

        approve_utilization_comment(comment_id, user['id'])
        approved_comment = get_registered_utilization_comment(utilization.id)[0]

        assert approved_comment.category == category
        assert approved_comment.content == content
        assert approved_comment.approval is True
        assert approved_comment.approved == datetime.now()
        assert approved_comment.approval_user_id == user['id']

    def test_get_utilization_comment_categories(self):
        assert get_utilization_comment_categories() == UtilizationCommentCategory

    def test_get_issue_resolutions(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])
        user = factories.User()

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        utilization_description = 'test description'

        register_utilization(
            utilization_id, resource['id'], title, url, utilization_description, False
        )

        utilization = get_registered_utilization(resource['id'])
        assert get_registered_issue_resolution(utilization.id) is None
        issue_resolution_description = 'test issue resolution description'
        time = datetime.now()

        session.add(
            IssueResolution(
                utilization_id=utilization.id,
                description=issue_resolution_description,
                created=time,
                creator_user_id=user['id'],
            )
        )

        issue_resolution = get_issue_resolutions(utilization.id)[0]

        assert issue_resolution.utilization_id == utilization.id
        assert issue_resolution.description == issue_resolution_description
        assert issue_resolution.created == time
        assert issue_resolution.creator_user_id == user['id']

    def test_create_issue_resolution(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])
        user = factories.Sysadmin()

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        utilization_description = 'test description'

        register_utilization(
            utilization_id, resource['id'], title, url, utilization_description, False
        )

        utilization = get_registered_utilization(resource['id'])
        issue_resolution_description = 'test_issue_resolution_description'
        create_issue_resolution(
            utilization.id, issue_resolution_description, user['id']
        )

        issue_resolution = (utilization.id, issue_resolution_description, user['id'])

        result = get_registered_issue_resolution(utilization.id)

        assert result == issue_resolution

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_refresh_utilization_comments(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])
        user = factories.Sysadmin()

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'

        created = datetime.now()
        approved = datetime.now()
        comment_id = str(uuid.uuid4())
        category = UtilizationCommentCategory.QUESTION
        content = 'test content'

        register_utilization(
            utilization_id, resource['id'], title, url, description, True
        )

        result = get_utilization(utilization_id)

        assert result.comment == 0

        register_utilization_comment(
            comment_id,
            utilization_id,
            category,
            content,
            created,
            True,
            approved,
            user['id'],
        )

        refresh_utilization_comments(utilization_id)

        result = get_utilization(utilization_id)

        assert result.comment == 1



from unittest.mock import MagicMock, patch

import ckan.lib.mailer
import ckan.tests.factories as factories
import pytest
from ckan import model
from ckan.common import config
from jinja2 import Environment, FileSystemLoader

from ckanext.feedback.services.common.send_mail import DEFAULT_TEMPLATE_DIR, send_email

engine = model.repo.session.get_bind()

template_dir = (
    '/srv/app/src_extensions/ckanext-feedback/ckanext/feedback/tests/services/common'
)
template_name = 'mail_template.txt'


def create_org_admin_user():
    user = factories.User()
    user_dict = user
    user_dict['name'] = 'test_user'
    user_dict['email'] = 'user@email.com'

    organization_dict = factories.Organization()
    organization = model.Group.get(organization_dict['id'])

    member = MagicMock()
    member.table_name = 'user'
    member.table_id = user['id']
    member.capacity = 'admin'
    member.group_id = organization.id
    member.group = organization
    return (user_dict, organization.id)


def mock_get_members(_, a):
    return [
        ('id', '_', '_'),
    ]


def mock_show_user(_, b):
    return {'name': 'test_user', 'email': 'user@email.com'}


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestSendMail:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()

    @patch('ckanext.feedback.services.common.send_mail.toolkit.get_action')
    @patch('ckanext.feedback.services.common.send_mail.toolkit.enqueue_job')
    def test_send_email(self, mock_enqueue_job, mock_get_action):
        (user, organization_id) = create_org_admin_user()
        config['ckan.feedback.notice.email.enable'] = True
        config['ckan.feedback.notice.email.template_directory'] = template_dir
        subject = 'test subject'

        mock_enqueue_job.side_effect = Exception("Mock Exception")
        mock_get_action.side_effect = [mock_get_members, mock_show_user]

        email_body = (
            Environment(loader=FileSystemLoader(template_dir))
            .get_template(template_name)
            .render()
        )

        send_email(template_name, organization_id, subject)

        mock_enqueue_job.assert_called_once_with(
            ckan.lib.mailer.mail_recipient,
            kwargs={
                'recipient_name': user['name'],
                'recipient_email': user['email'],
                'subject': subject,
                'body': email_body,
            },
        )

    @patch('ckanext.feedback.services.common.send_mail.log.info')
    def test_send_email_disable(self, mock_log_info):
        config['ckan.feedback.notice.email.enable'] = False
        send_email(template_name, '', '')
        mock_log_info.assert_called_once_with('email notification is disabled.')

    @patch('ckanext.feedback.services.common.send_mail.log.error')
    def test_send_email_no_template(self, mock_log_error):
        config['ckan.feedback.notice.email.enable'] = True

        send_email('no_template.test', '', '')
        mock_log_error.assert_called_once_with(
            'template_file error. %s/%s: No such file or directory',
            DEFAULT_TEMPLATE_DIR,
            'no_template.test',
        )

    @patch('ckanext.feedback.services.common.send_mail.toolkit.get_action')
    @patch('ckanext.feedback.services.common.send_mail.toolkit.enqueue_job')
    def test_send_email_no_subject(self, mock_enqueue_job, mock_get_action):
        (user, organization_id) = create_org_admin_user()
        config['ckan.feedback.notice.email.enable'] = True
        config['ckan.feedback.notice.email.template_directory'] = template_dir

        mock_get_action.side_effect = [mock_get_members, mock_show_user]

        email_body = (
            Environment(loader=FileSystemLoader(template_dir))
            .get_template(template_name)
            .render()
        )

        send_email(template_name, organization_id, '')

        mock_enqueue_job.assert_called_once_with(
            ckan.lib.mailer.mail_recipient,
            kwargs={
                'recipient_name': user['name'],
                'recipient_email': user['email'],
                'subject': 'New Submission Notification',
                'body': email_body,
            },
        )



import datetime
import logging
import uuid

from ckan.model import Resource
from sqlalchemy import func

from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.session import session

log = logging.getLogger(__name__)


def get_package_downloads(package_id):
    count = (
        session.query(func.sum(DownloadSummary.download))
        .join(Resource)
        .filter(Resource.package_id == package_id)
        .scalar()
    )
    return count or 0


def get_resource_downloads(resource_id):
    count = (
        session.query(DownloadSummary.download)
        .filter(DownloadSummary.resource_id == resource_id)
        .scalar()
    )
    return count or 0


def increment_resource_downloads(resource_id):
    download_summary = (
        session.query(DownloadSummary)
        .filter(DownloadSummary.resource_id == resource_id)
        .first()
    )
    if download_summary is None:
        download_summary = DownloadSummary(
            id=str(uuid.uuid4()),
            resource_id=resource_id,
            download=1,
            created=datetime.datetime.now(),
            updated=datetime.datetime.now(),
        )
        session.add(download_summary)
    else:
        download_summary.download = download_summary.download + 1
        download_summary.updated = datetime.datetime.now()
    session.commit()



from datetime import datetime

from ckan.model.group import Group
from ckan.model.resource import Resource
from sqlalchemy import func

from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentSummary,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization, UtilizationComment


# Get approval utilization comment count using utilization.id
def get_utilization_comments(utilization_id):
    count = (
        session.query(UtilizationComment)
        .filter(
            UtilizationComment.utilization_id == utilization_id,
            UtilizationComment.approval,
        )
        .count()
    )
    return count


# Get utilizations using comment_id_list
def get_utilizations(comment_id_list):
    utilizations = (
        session.query(Utilization)
        .join(UtilizationComment)
        .filter(UtilizationComment.id.in_(comment_id_list))
    ).all()
    return utilizations


# Get organization using owner_org
def get_organization(owner_org):
    organization = session.query(Group).filter(Group.id == owner_org).first()
    return organization


# Recalculate total approved bulk utilizations comments
def refresh_utilizations_comments(utilizations):
    session.bulk_update_mappings(
        Utilization,
        [
            {
                'id': utilization.id,
                'comment': get_utilization_comments(utilization.id),
                'updated': datetime.now(),
            }
            for utilization in utilizations
        ],
    )


# Get resource comment summaries using comment_id_list
def get_resource_comment_summaries(comment_id_list):
    resource_comment_summaries = (
        session.query(ResourceCommentSummary)
        .join(Resource)
        .join(ResourceComment)
        .filter(ResourceComment.id.in_(comment_id_list))
    ).all()
    return resource_comment_summaries


# Recalculate total approved bulk resources comments
def refresh_resources_comments(resource_comment_summaries):
    mappings = []
    for resource_comment_summary in resource_comment_summaries:
        row = (
            session.query(
                func.sum(ResourceComment.rating).label('total_rating'),
                func.count().label('total_comment'),
                func.count(ResourceComment.rating).label('total_rating_comment'),
            )
            .filter(
                ResourceComment.resource_id == resource_comment_summary.resource.id,
                ResourceComment.approval,
            )
            .first()
        )
        if row.total_rating is None:
            rating = 0
        else:
            rating = row.total_rating / row.total_rating_comment
        mappings.append(
            {
                'id': resource_comment_summary.id,
                'comment': row.total_comment,
                'rating_comment': row.total_rating_comment,
                'rating': rating,
                'updated': datetime.now(),
            }
        )
    session.bulk_update_mappings(ResourceCommentSummary, mappings)


# Approve selected utilization comments
def approve_utilization_comments(comment_id_list, approval_user_id):
    session.bulk_update_mappings(
        UtilizationComment,
        [
            {
                'id': comment_id,
                'approval': True,
                'approved': datetime.now(),
                'approval_user_id': approval_user_id,
            }
            for comment_id in comment_id_list
        ],
    )


# Delete selected utilization comments
def delete_utilization_comments(comment_id_list):
    (
        session.query(UtilizationComment)
        .filter(UtilizationComment.id.in_(comment_id_list))
        .delete(synchronize_session='fetch')
    )


# Approve selected resource comments
def approve_resource_comments(comment_id_list, approval_user_id):
    session.bulk_update_mappings(
        ResourceComment,
        [
            {
                'id': comment_id,
                'approval': True,
                'approved': datetime.now(),
                'approval_user_id': approval_user_id,
            }
            for comment_id in comment_id_list
        ],
    )


# Delete selected resource comments
def delete_resource_comments(comment_id_list):
    (
        session.query(ResourceComment)
        .filter(ResourceComment.id.in_(comment_id_list))
        .delete(synchronize_session='fetch')
    )



import logging

from psycopg2.errors import UndefinedTable
from sqlalchemy.exc import ProgrammingError

from ckanext.feedback.models.session import session

log = logging.getLogger(__name__)


def add_error_handler(func):
    def wrapper():
        blueprint = func()

        @blueprint.app_errorhandler(ProgrammingError)
        def handle_programming_error(e):
            if isinstance(e.orig, UndefinedTable):
                log.error(
                    'Some tables does not exit.'
                    ' Run "ckan --config=/etc/ckan/production.ini feedback init".'
                )

            session.rollback()
            raise e

        @blueprint.app_errorhandler(Exception)
        def handle_exception(e):
            session.rollback()
            raise e

        return blueprint

    return wrapper



from unittest.mock import patch

import pytest
from ckan import model
from ckan.common import _
from ckan.model import User
from ckan.tests import factories
from flask import Flask, g

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.services.common.check import (
    check_administrator,
    has_organization_admin_role,
    is_organization_admin,
)

engine = model.repo.session.get_bind()


@pytest.fixture
def sysadmin_env():
    user = factories.SysadminWithToken()
    env = {'Authorization': user['token']}
    return env


@pytest.fixture
def user_env():
    user = factories.UserWithToken()
    env = {'Authorization': user['token']}
    return env


def mock_current_user(current_user, user):
    user_obj = model.User.get(user['name'])
    # mock current_user
    current_user.return_value = user_obj


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestCheck:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)

    @patch('flask_login.utils._get_user')
    def test_check_administrator(self, current_user, app, sysadmin_env):
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        @check_administrator
        def dummy_function():
            return 'function is called'

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            result = dummy_function()

        assert result == 'function is called'

    @patch('flask_login.utils._get_user')
    def test_check_administrator_with_org_admin_user(self, current_user, app, user_env):
        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)

        organization_dict = factories.Organization()
        organization = model.Group.get(organization_dict['id'])
        member = model.Member(
            group=organization,
            group_id=organization.id,
            table_id=user.id,
            capacity='admin',
            table_name='user',
        )
        model.Session.add(member)
        model.Session.commit()

        @check_administrator
        def dummy_function():
            return 'function is called'

        with app.get(url='/', environ_base=user_env):
            g.userobj = current_user
            result = dummy_function()

        assert result == 'function is called'

    @patch('ckanext.feedback.services.common.check.toolkit')
    def test_check_administrator_without_user(self, mock_toolkit):
        @check_administrator
        def dummy_function():
            return 'function is called'

        g.userobj = None
        dummy_function()

        mock_toolkit.abort.assert_called_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the'
                ' URL manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.services.common.check.toolkit')
    def test_check_administrator_with_user(self, mock_toolkit, current_user):
        @check_administrator
        def dummy_function():
            return 'function is called'

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user
        dummy_function()

        mock_toolkit.abort.assert_called_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the'
                ' URL manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    def test_is_organization_admin_with_user(self, current_user, app, user_env):
        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)

        organization_dict = factories.Organization()
        organization = model.Group.get(organization_dict['id'])
        member = model.Member(
            group=organization,
            group_id=organization.id,
            table_id=user.id,
            capacity='admin',
            table_name='user',
        )
        model.Session.add(member)
        model.Session.commit()
        with app.get(url='/', environ_base=user_env):
            g.userobj = current_user
            result = is_organization_admin()

        assert result is True

    def test_is_organization_admin_without_user(self):
        g.userobj = None
        result = is_organization_admin()

        assert result is False

    @patch('flask_login.utils._get_user')
    def test_has_organization_admin_role_with_user(self, current_user, app, user_env):
        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)

        organization_dict1 = factories.Organization()
        organization_dict2 = factories.Organization()
        organization_dict3 = factories.Organization()
        organization1 = model.Group.get(organization_dict1['id'])
        organization2 = model.Group.get(organization_dict2['id'])
        organization3 = model.Group.get(organization_dict3['id'])
        member1 = model.Member(
            group=organization1,
            group_id=organization1.id,
            table_id=user.id,
            capacity='admin',
            table_name='user',
        )
        model.Session.add(member1)
        member3 = model.Member(
            group=organization1,
            group_id=organization3.id,
            table_id=user.id,
            capacity='member',
            table_name='user',
        )
        model.Session.add(member1)
        model.Session.add(member3)
        model.Session.commit()
        with app.get(url='/', environ_base=user_env):
            g.userobj = current_user
            assert has_organization_admin_role(organization1.id) is True
            assert has_organization_admin_role(organization2.id) is False
            assert has_organization_admin_role(organization3.id) is False

    def test_has_organization_admin_role_without_user(self):
        organization_dict1 = factories.Organization()
        organization1 = model.Group.get(organization_dict1['id'])

        g.userobj = None
        result = has_organization_admin_role(organization1.id)

        assert result is False



from ckan.model.resource import Resource
from sqlalchemy import TIMESTAMP, Column, ForeignKey, Integer, Text
from sqlalchemy.orm import relationship

from ckanext.feedback.models.session import Base


class DownloadSummary(Base):
    __tablename__ = 'download_summary'
    id = Column(Text, primary_key=True, nullable=False)
    resource_id = Column(
        Text,
        ForeignKey('resource.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    download = Column(Integer)
    created = Column(TIMESTAMP)
    updated = Column(TIMESTAMP)

    resource = relationship(Resource)



import ckan.tests.factories as factories
import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization
from ckanext.feedback.services.utilization.registration import create_utilization


def get_utilization(resource_id):
    return (
        session.query(
            Utilization.title,
            Utilization.url,
            Utilization.description,
        )
        .filter(Utilization.resource_id == resource_id)
        .first()
    )


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_create_utilization(self):
        resource = factories.Resource()

        title = 'test title'
        url = 'test url'
        description = 'test description'

        assert get_utilization(resource['id']) is None

        create_utilization(resource['id'], title, url, description)

        result = get_utilization(resource['id'])

        assert result.title == title
        assert result.url == url
        assert result.description == description



from ckan.common import _, current_user, g, request
from ckan.lib import helpers
from ckan.plugins import toolkit
from flask import redirect, url_for

import ckanext.feedback.services.management.comments as comments_service
import ckanext.feedback.services.resource.comment as resource_comment_service
import ckanext.feedback.services.utilization.details as utilization_detail_service
from ckanext.feedback.models.session import session
from ckanext.feedback.services.common.check import (
    check_administrator,
    has_organization_admin_role,
)


class ManagementController:
    # management/comments
    @staticmethod
    @check_administrator
    def comments():
        tab = request.args.get('tab', 'utilization-comments')
        categories = utilization_detail_service.get_utilization_comment_categories()

        # If user is organization admin
        if not current_user.sysadmin:
            ids = current_user.get_group_ids(
                group_type='organization', capacity='admin'
            )
            resource_comments = resource_comment_service.get_resource_comments(
                owner_orgs=ids
            )
            utilization_comments = utilization_detail_service.get_utilization_comments(
                owner_orgs=ids
            )
            g.pkg_dict = {
                'organization': {
                    'name': current_user.get_groups(group_type='organization')[0].name,
                }
            }
        else:
            resource_comments = resource_comment_service.get_resource_comments()
            utilization_comments = utilization_detail_service.get_utilization_comments()
        return toolkit.render(
            'management/comments.html',
            {
                'categories': categories,
                'utilization_comments': utilization_comments,
                'resource_comments': resource_comments,
                'tab': tab,
            },
        )

    # management/approve_bulk_utilization_comments
    @staticmethod
    @check_administrator
    def approve_bulk_utilization_comments():
        comments = request.form.getlist('utilization-comments-checkbox')
        if comments:
            utilizations = comments_service.get_utilizations(comments)
            ManagementController._check_organization_admin_role_with_utilization(
                utilizations
            )
            comments_service.approve_utilization_comments(comments, current_user.id)
            comments_service.refresh_utilizations_comments(utilizations)
            session.commit()
            helpers.flash_success(
                f'{len(comments)} ' + _('bulk approval completed.'),
                allow_html=True,
            )
        return redirect(url_for('management.comments', tab='utilization-comments'))

    # management/approve_bulk_resource_comments
    @staticmethod
    @check_administrator
    def approve_bulk_resource_comments():
        comments = request.form.getlist('resource-comments-checkbox')
        if comments:
            resource_comment_summaries = (
                comments_service.get_resource_comment_summaries(comments)
            )
            ManagementController._check_organization_admin_role_with_resource(
                resource_comment_summaries
            )
            comments_service.approve_resource_comments(comments, current_user.id)
            comments_service.refresh_resources_comments(resource_comment_summaries)
            session.commit()
            helpers.flash_success(
                f'{len(comments)} ' + _('bulk approval completed.'),
                allow_html=True,
            )
        return redirect(url_for('management.comments', tab='resource-comments'))

    # management/delete_bulk_utilization_comments
    @staticmethod
    @check_administrator
    def delete_bulk_utilization_comments():
        comments = request.form.getlist('utilization-comments-checkbox')
        if comments:
            utilizations = comments_service.get_utilizations(comments)
            ManagementController._check_organization_admin_role_with_utilization(
                utilizations
            )
            comments_service.delete_utilization_comments(comments)
            comments_service.refresh_utilizations_comments(utilizations)
            session.commit()

            helpers.flash_success(
                f'{len(comments)} ' + _('bulk delete completed.'),
                allow_html=True,
            )
        return redirect(url_for('management.comments', tab='utilization-comments'))

    # management/delete_bulk_resource_comments
    @staticmethod
    @check_administrator
    def delete_bulk_resource_comments():
        comments = request.form.getlist('resource-comments-checkbox')
        if comments:
            resource_comment_summaries = (
                comments_service.get_resource_comment_summaries(comments)
            )
            ManagementController._check_organization_admin_role_with_resource(
                resource_comment_summaries
            )
            comments_service.delete_resource_comments(comments)
            comments_service.refresh_resources_comments(resource_comment_summaries)
            session.commit()

            helpers.flash_success(
                f'{len(comments)} ' + _('bulk delete completed.'),
                allow_html=True,
            )
        return redirect(url_for('management.comments', tab='resource-comments'))

    @staticmethod
    def _check_organization_admin_role_with_utilization(utilizations):
        for utilization in utilizations:
            if (
                not has_organization_admin_role(utilization.resource.package.owner_org)
                and not current_user.sysadmin
            ):
                toolkit.abort(
                    404,
                    _(
                        'The requested URL was not found on the server. If you entered'
                        ' the URL manually please check your spelling and try again.'
                    ),
                )

    @staticmethod
    def _check_organization_admin_role_with_resource(resource_comment_summaries):
        for resource_comment_summary in resource_comment_summaries:
            if (
                not has_organization_admin_role(
                    resource_comment_summary.resource.package.owner_org
                )
                and not current_user.sysadmin
            ):
                toolkit.abort(
                    404,
                    _(
                        'The requested URL was not found on the server. If you entered'
                        ' the URL manually please check your spelling and try again.'
                    ),
                )



import logging
import os
from urllib.parse import urlparse

import ckan.views.resource as resource
import requests
from flask import Response, request

from ckanext.feedback.services.common import config as feedback_config
from ckanext.feedback.services.download.summary import increment_resource_downloads
from ckanext.feedback.services.resource.comment import get_resource

log = logging.getLogger(__name__)


class DownloadController:
    # extend default download function to count when a resource is downloaded
    @staticmethod
    def extended_download(package_type, id, resource_id, filename=None):
        if filename is None:
            filename = get_resource(resource_id).Resource.url

        if (
            request.headers.get('Sec-Fetch-Dest') == 'document'
            or request.args.get('user-download') == 'true'
        ):
            increment_resource_downloads(resource_id)

        handler = feedback_config.download_handler()
        if not handler:
            log.debug('Use default CKAN callback for resource.download')
            handler = resource.download
        response = handler(
            package_type=package_type,
            id=id,
            resource_id=resource_id,
            filename=filename,
        )

        if response.status_code == 302:
            url = response.headers.get('Location')
            log.debug(f"download to redirect URL.[{url}]")
            filename = os.path.basename(urlparse(url).path)
            try:
                redirect_response = requests.get(url, allow_redirects=True)
                external_response = Response(
                    redirect_response.content,
                    headers=redirect_response.headers.__dict__,
                    content_type=redirect_response.headers['Content-Type'],
                )
            except requests.exceptions.ConnectionError:
                log.exception(f'Can not connect to external resource. URL[{url}]')
                return response
            if external_response.status_code != 200:
                log.exception(f'Failure to acquire external resource. URL[{url}]')
                return response
            response = external_response

        c_d_value = response.headers.get('Content-Disposition')
        if c_d_value:
            c_d_value = c_d_value.replace('inline', 'attachment')
        else:
            c_d_value = 'attachment'
        if 'filename' not in c_d_value:
            c_d_value = f'{c_d_value}; filename="{filename}"'
        response.headers.set('Content-Disposition', c_d_value)

        return response



from datetime import datetime

from ckan.model.group import Group
from ckan.model.package import Package
from ckan.model.resource import Resource
from flask import request

from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentCategory,
    ResourceCommentReply,
)
from ckanext.feedback.models.session import session


# Get resource from the selected resource_id
def get_resource(resource_id):
    return (
        session.query(
            Resource,
            Package.id.label('organization_id'),
            Group.name.label('organization_name'),
        )
        .join(Package)
        .join(Group, Package.owner_org == Group.id)
        .filter(Resource.id == resource_id)
        .first()
    )


# Get comments related to the dataset or resource
def get_resource_comments(
    resource_id=None, approval=None, owner_orgs=None, limit=None, offset=None
):
    query = session.query(ResourceComment).order_by(ResourceComment.created.desc())
    if resource_id is not None:
        query = query.filter(ResourceComment.resource_id == resource_id)
    if approval is not None:
        query = query.filter(ResourceComment.approval == approval)
    if owner_orgs is not None:
        query = (
            query.join(Resource).join(Package).filter(Package.owner_org.in_(owner_orgs))
        )

    results = query.limit(limit).offset(offset).all()
    if limit is not None or offset is not None:
        total_count = query.count()
        return results, total_count
    return results


# Get category enum names and values
def get_resource_comment_categories():
    return ResourceCommentCategory


# Create new comment
def create_resource_comment(resource_id, category, content, rating):
    comment = ResourceComment(
        resource_id=resource_id,
        category=category,
        content=content,
        rating=rating,
    )
    session.add(comment)


# Approve selected resource comment
def approve_resource_comment(resource_comment_id, approval_user_id):
    comment = session.query(ResourceComment).get(resource_comment_id)
    comment.approval = True
    comment.approved = datetime.now()
    comment.approval_user_id = approval_user_id


# Get reply for target comment
def get_comment_reply(resource_comment_id):
    return (
        session.query(ResourceCommentReply)
        .filter(ResourceCommentReply.resource_comment_id == resource_comment_id)
        .first()
    )


# Create new reply
def create_reply(resource_comment_id, content, creator_user_id):
    reply = ResourceCommentReply(
        resource_comment_id=resource_comment_id,
        content=content,
        creator_user_id=creator_user_id,
    )
    session.add(reply)


# Get cookie
def get_cookie(resource_id):
    return request.cookies.get(resource_id)



from unittest.mock import call, patch

import pytest
from ckan import model
from ckan.common import _
from ckan.logic import get_action
from ckan.model import User
from ckan.tests import factories
from flask import Flask, g

import ckanext.feedback.services.resource.comment as comment_service
from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.controllers.resource import ResourceController
from ckanext.feedback.models.session import session

engine = model.repo.session.get_bind()


@pytest.fixture
def sysadmin_env():
    user = factories.SysadminWithToken()
    env = {'Authorization': user['token']}
    return env


@pytest.fixture
def user_env():
    user = factories.UserWithToken()
    env = {'Authorization': user['token']}
    return env


def mock_current_user(current_user, user):
    user_obj = model.User.get(user['name'])
    # mock current_user
    current_user.return_value = user_obj


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestResourceController:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.utilization.get_pagination_value')
    @patch('ckanext.feedback.controllers.utilization.helpers.Page')
    @patch('ckanext.feedback.controllers.resource.toolkit.render')
    @patch('ckanext.feedback.controllers.resource.request')
    def test_comment_with_sysadmin(
        self,
        mock_request,
        mock_render,
        mock_page,
        mock_pagination,
        current_user,
        app,
        sysadmin_env,
    ):
        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)

        organization_dict = factories.Organization(
            name='org_name',
        )
        package = factories.Dataset(owner_org=organization_dict['id'])
        resource = factories.Resource(package_id=package['id'])
        resource['package'] = package
        # session.commit()
        resource_id = resource['id']

        page = 1
        limit = 20
        offset = 0
        _ = ''

        mock_pagination.return_value = [
            page,
            limit,
            offset,
            _,
        ]

        mock_page.return_value = 'mock_page'

        with app.get(url='/', environ_base=sysadmin_env):
            g.userobj = current_user
            ResourceController.comment(resource_id)

        approval = None
        resource = comment_service.get_resource(resource_id)
        comments, total_count = comment_service.get_resource_comments(
            resource_id,
            approval,
            limit=limit,
            offset=offset,
        )
        categories = comment_service.get_resource_comment_categories()
        cookie = comment_service.get_cookie(resource_id)
        context = {'model': model, 'session': session, 'for_view': True}
        package = get_action('package_show')(
            context, {'id': resource.Resource.package_id}
        )

        mock_page.assert_called_once_with(
            collection=comments,
            page=page,
            item_count=total_count,
            items_per_page=limit,
        )

        assert g.pkg_dict["organization"]['name'] == 'org_name'
        mock_render.assert_called_once_with(
            'resource/comment.html',
            {
                'resource': resource.Resource,
                'pkg_dict': package,
                'categories': categories,
                'cookie': cookie,
                'selected_category': '',
                'content': '',
                'page': 'mock_page',
            },
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.utilization.get_pagination_value')
    @patch('ckanext.feedback.controllers.utilization.helpers.Page')
    @patch('ckanext.feedback.controllers.resource.toolkit.render')
    @patch('ckanext.feedback.controllers.resource.request')
    def test_comment_with_user(
        self,
        mock_request,
        mock_render,
        mock_page,
        mock_pagination,
        current_user,
        app,
        user_env,
    ):
        user_dict = factories.User()
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        mock_current_user(current_user, user_dict)
        resource = factories.Resource(package_id=dataset['id'])
        resource_id = resource['id']

        page = 1
        limit = 20
        offset = 0
        _ = ''

        mock_pagination.return_value = [
            page,
            limit,
            offset,
            _,
        ]

        mock_page.return_value = 'mock_page'

        with app.get(url='/', environ_base=user_env):
            g.userobj = current_user
            ResourceController.comment(resource_id)

        approval = True
        resource = comment_service.get_resource(resource_id)
        comments, total_count = comment_service.get_resource_comments(
            resource_id,
            approval,
            limit=limit,
            offset=offset,
        )
        categories = comment_service.get_resource_comment_categories()
        cookie = comment_service.get_cookie(resource_id)
        context = {'model': model, 'session': session, 'for_view': True}
        package = get_action('package_show')(
            context, {'id': resource.Resource.package_id}
        )

        mock_page.assert_called_once_with(
            collection=comments,
            page=page,
            item_count=total_count,
            items_per_page=limit,
        )

        mock_render.assert_called_once_with(
            'resource/comment.html',
            {
                'resource': resource.Resource,
                'pkg_dict': package,
                'categories': categories,
                'cookie': cookie,
                'selected_category': '',
                'content': '',
                'page': 'mock_page',
            },
        )

    @patch('ckanext.feedback.controllers.utilization.get_pagination_value')
    @patch('ckanext.feedback.controllers.utilization.helpers.Page')
    @patch('ckanext.feedback.controllers.resource.toolkit.render')
    @patch('ckanext.feedback.controllers.resource.request')
    def test_comment_without_user(
        self, mock_request, mock_render, mock_page, mock_pagination, app
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        resource_id = resource['id']

        page = 1
        limit = 20
        offset = 0
        _ = ''

        mock_pagination.return_value = [
            page,
            limit,
            offset,
            _,
        ]

        mock_page.return_value = 'mock_page'

        with app.get(url='/'):
            g.userobj = None
            ResourceController.comment(resource_id)

        approval = False
        resource = comment_service.get_resource(resource_id)
        comments, total_count = comment_service.get_resource_comments(
            resource_id,
            approval,
            limit=limit,
            offset=offset,
        )
        categories = comment_service.get_resource_comment_categories()
        cookie = comment_service.get_cookie(resource_id)
        context = {'model': model, 'session': session, 'for_view': True}
        package = get_action('package_show')(
            context, {'id': resource.Resource.package_id}
        )

        mock_page.assert_called_once_with(
            collection=comments,
            page=page,
            item_count=total_count,
            items_per_page=limit,
        )

        mock_render.assert_called_once_with(
            'resource/comment.html',
            {
                'resource': resource.Resource,
                'pkg_dict': package,
                'categories': categories,
                'cookie': cookie,
                'selected_category': '',
                'content': '',
                'page': 'mock_page',
            },
        )

    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.helpers.flash_success')
    @patch('ckanext.feedback.controllers.resource.session.commit')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    @patch('ckanext.feedback.controllers.resource.make_response')
    @patch('ckanext.feedback.controllers.resource.send_email')
    def test_create_comment(
        self,
        mock_send_email,
        mock_make_response,
        mock_redirect,
        mock_url_for,
        mock_session_commit,
        mock_flash_success,
        mock_comment_service,
        mock_summary_service,
        mock_form,
    ):
        resource_id = 'resource id'
        category = 'category'
        comment_content = 'content'
        rating = '1'
        package_name = 'ota'
        mock_form.get.side_effect = [
            package_name,
            comment_content,
            comment_content,
            category,
            rating,
            rating,
        ]
        mock_send_email.side_effect = Exception("Mock Exception")

        mock_url_for.return_value = 'resource comment'
        resp = ResourceController.create_comment(resource_id)

        mock_comment_service.create_resource_comment.assert_called_once_with(
            resource_id, category, comment_content, int(rating)
        )
        mock_summary_service.create_resource_summary.assert_called_once_with(
            resource_id
        )
        mock_session_commit.assert_called_once()
        mock_flash_success.assert_called_once()
        mock_url_for.assert_has_calls(
            [
                call(
                    'resource_comment.comment', resource_id=resource_id, _external=True
                ),
                call('resource.read', id=package_name, resource_id=resource_id),
            ]
        )
        mock_redirect.assert_called_once_with('resource comment')
        mock_make_response.assert_called_once_with(mock_redirect())
        resp.set_cookie.assert_called_once_with(resource_id, 'alreadyPosted')

    @patch('ckanext.feedback.controllers.resource.validate_service.validate_comment')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.helpers.flash_success')
    @patch('ckanext.feedback.controllers.resource.session.commit')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    @patch('ckanext.feedback.controllers.resource.make_response')
    def test_create_comment_without_category_content(
        self,
        mock_make_response,
        mock_redirect,
        mock_url_for,
        mock_session_commit,
        mock_flash_success,
        mock_comment_service,
        mock_summary_service,
        mock_form,
        mock_toolkit_abort,
        mock_validate_comment,
    ):
        resource_id = 'resource id'
        mock_form.get.return_value = None
        mock_validate_comment.return_value = None

        ResourceController.create_comment(resource_id)
        mock_toolkit_abort.assert_called_once_with(400)

    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.ResourceController.comment')
    @patch('ckanext.feedback.controllers.resource.toolkit.redirect_to')
    @patch('ckanext.feedback.controllers.resource.helpers.flash_error')
    def test_create_comment_without_comment_length(
        self,
        mock_flash_flash_error,
        mock_redirect_to,
        mock_comment,
        mock_form,
    ):
        resource_id = 'resource id'
        category = 'category'
        content = 'ex'
        while True:
            content += content
            if 1000 < len(content):
                break

        mock_form.get.side_effect = [
            '',
            content,
            content,
            category,
            None,
        ]
        ResourceController.create_comment(resource_id)

        mock_flash_flash_error.assert_called_once_with(
            'Please keep the comment length below 1000',
            allow_html=True,
        )
        mock_comment.assert_called_once_with(resource_id, category, content)

    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.ResourceController.comment')
    @patch('ckanext.feedback.controllers.resource.is_recaptcha_verified')
    @patch('ckanext.feedback.controllers.resource.helpers.flash_error')
    def test_create_comment_without_bad_recaptcha(
        self,
        mock_flash_error,
        mock_is_recaptcha_verified,
        mock_comment,
        mock_form,
    ):
        resource_id = 'resource_id'
        comment_content = 'comment_content'
        category = 'category'
        package_name = 'ota'
        mock_form.get.side_effect = [
            package_name,
            comment_content,
            comment_content,
            category,
            None,
        ]

        mock_is_recaptcha_verified.return_value = False
        ResourceController.create_comment(resource_id)
        mock_comment.assert_called_once_with(resource_id, category, comment_content)

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.session.commit')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    def test_approve_comment_with_sysadmin(
        self,
        mock_redirect,
        mock_url_for,
        mock_session_commit,
        mock_comment_service,
        mock_summary_service,
        mock_form,
        current_user,
    ):
        resource_id = 'resource id'
        resource_comment_id = 'resource comment id'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        mock_form.get.side_effect = [resource_comment_id]

        mock_url_for.return_value = 'resource comment url'
        ResourceController.approve_comment(resource_id)

        mock_comment_service.approve_resource_comment.assert_called_once_with(
            resource_comment_id, user_dict['id']
        )
        mock_summary_service.refresh_resource_summary.assert_called_once_with(
            resource_id
        )
        mock_session_commit.assert_called_once()
        mock_url_for.assert_called_once_with(
            'resource_comment.comment', resource_id=resource_id
        )

        mock_redirect.assert_called_once_with('resource comment url')

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    def test_approve_comment_with_user(self, mock_toolkit_abort, current_user):
        resource_id = 'resource id'

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        ResourceController.approve_comment(resource_id)
        mock_toolkit_abort.assert_called_once_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    def test_approve_comment_with_other_organization_admin_user(
        self,
        mock_redirect,
        mock_url_for,
        mock_comment_service,
        mock_toolkit_abort,
        current_user,
    ):
        organization_dict = factories.Organization()
        package = factories.Dataset(owner_org=organization_dict['id'])
        resource = factories.Resource(package_id=package['id'])

        dummy_organization_dict = factories.Organization()
        dummy_organization = model.Group.get(dummy_organization_dict['id'])

        user_dict = factories.User()
        user = User.get(user_dict['id'])
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        member = model.Member(
            group=dummy_organization,
            group_id=dummy_organization_dict['id'],
            table_id=user.id,
            table_name='user',
            capacity='admin',
        )
        model.Session.add(member)
        model.Session.commit()

        ResourceController.approve_comment(resource['id'])
        mock_toolkit_abort.assert_any_call(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.request.form')
    def test_approve_comment_without_resource_comment_id(
        self,
        mock_form,
        mock_comment_service,
        mock_summary_service,
        mock_toolkit_abort,
        current_user,
    ):
        resource_id = 'resource id'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        mock_form.get.side_effect = [None]

        ResourceController.approve_comment(resource_id)
        mock_toolkit_abort.assert_called_once_with(400)

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.request.form')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.session.commit')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    def test_reply_with_sysadmin(
        self,
        mock_redirect,
        mock_url_for,
        mock_session_commit,
        mock_comment_service,
        mock_summary_service,
        mock_form,
        current_user,
    ):
        resource_id = 'resource id'
        resource_comment_id = 'resource comment id'
        reply_content = 'reply content'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        mock_form.get.side_effect = [
            resource_comment_id,
            reply_content,
        ]

        mock_url_for.return_value = 'resource comment url'
        ResourceController.reply(resource_id)

        mock_comment_service.create_reply.assert_called_once_with(
            resource_comment_id, reply_content, user_dict['id']
        )
        mock_session_commit.assert_called_once()
        mock_url_for.assert_called_once_with(
            'resource_comment.comment', resource_id=resource_id
        )

        mock_redirect.assert_called_once_with('resource comment url')

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    def test_reply_with_user(self, mock_toolkit_abort, current_user):
        resource_id = 'resource id'

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        ResourceController.reply(resource_id)
        mock_toolkit_abort.assert_called_once_with(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.url_for')
    @patch('ckanext.feedback.controllers.resource.redirect')
    def test_reply_with_other_organization_admin_user(
        self,
        mock_redirect,
        mock_url_for,
        mock_comment_service,
        mock_toolkit_abort,
        current_user,
    ):
        organization_dict = factories.Organization()
        package = factories.Dataset(owner_org=organization_dict['id'])
        resource = factories.Resource(package_id=package['id'])

        dummy_organization_dict = factories.Organization()
        dummy_organization = model.Group.get(dummy_organization_dict['id'])

        user_dict = factories.User()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        member = model.Member(
            group=dummy_organization,
            group_id=dummy_organization_dict['id'],
            table_id=user_dict['id'],
            table_name='user',
            capacity='admin',
        )
        model.Session.add(member)
        model.Session.commit()

        ResourceController.reply(resource['id'])
        mock_toolkit_abort.assert_any_call(
            404,
            _(
                'The requested URL was not found on the server. If you entered the URL'
                ' manually please check your spelling and try again.'
            ),
        )

    @patch('flask_login.utils._get_user')
    @patch('ckanext.feedback.controllers.resource.toolkit.abort')
    @patch('ckanext.feedback.controllers.resource.summary_service')
    @patch('ckanext.feedback.controllers.resource.comment_service')
    @patch('ckanext.feedback.controllers.resource.request.form')
    def test_reply_without_resource_comment_id(
        self,
        mock_form,
        mock_comment_service,
        mock_summary_service,
        mock_toolkit_abort,
        current_user,
    ):
        resource_id = 'resource id'

        user_dict = factories.Sysadmin()
        mock_current_user(current_user, user_dict)
        g.userobj = current_user

        mock_form.get.side_effect = [
            None,
            None,
        ]

        ResourceController.reply(resource_id)
        mock_toolkit_abort.assert_called_once_with(400)



import logging
import os

import ckan.lib.mailer
import ckan.plugins.toolkit as toolkit
from ckan.common import config
from jinja2 import Environment, FileSystemLoader

log = logging.getLogger(__name__)
DEFAULT_TEMPLATE_DIR = (
    '/srv/app'
    + '/src_extensions/ckanext-feedback/ckanext/feedback/templates/email_notification'
)


def send_email(template_name, organization_id, subject, **kwargs):
    if not toolkit.asbool(config.get('ckan.feedback.notice.email.enable', False)):
        log.info('email notification is disabled.')
        return

    # settings email_template and subject from [feedback_config.json > ckan.ini]
    template_dir = config.get('ckan.feedback.notice.email.template_directory')
    if not os.path.isfile(f'{template_dir}/{template_name}'):
        template_dir = DEFAULT_TEMPLATE_DIR

    if not os.path.isfile(f'{template_dir}/{template_name}'):
        log.error(
            'template_file error. %s/%s: No such file or directory',
            template_dir,
            template_name,
        )
        return

    log.info('use template. %s/%s', template_dir, template_name)

    if not subject:
        subject = 'New Submission Notification'
        log.info('use default_subject: [%s]', subject)

    email_body = (
        Environment(loader=FileSystemLoader(template_dir))
        .get_template(template_name)
        .render(kwargs)
    )

    # Retrieving organization administrators and sending emails
    get_members = toolkit.get_action('member_list')
    show_user = toolkit.get_action('user_show')

    condition = {'id': organization_id, 'object_type': 'user', 'capacity': 'admin'}
    users = [show_user(None, {'id': id}) for (id, _, _) in get_members(None, condition)]

    for user in users:
        try:
            toolkit.enqueue_job(
                ckan.lib.mailer.mail_recipient,
                kwargs={
                    'recipient_name': user['name'],
                    'recipient_email': user['email'],
                    'subject': subject,
                    'body': email_body,
                },
            )
        except Exception:
            log.exception(f'user: {user}')
